"""
    Name: configuration.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: A module to load config file.
    Coding: UTF-8
"""

import json
from typing import Dict


def load_config(source: str) -> Dict:
    """Load Configuration File

    Reads the configuration file and returns it as a dictionary.

    Args:
        source: Configuration file path.

    Returns:
        A dict of config information.

        For example:
        {'testcase_num': '3',
        'testcase_dir': 'example/testdata',
        'submit_path': 'example/submit/*',
        'exec_path': 'example/exec/student.exec'}

    Raises:
        IOError: A error occurred when missing config file.
    """
    f = open(source, encoding='utf-8')
    return json.load(f)



import time
import unittest
import pygrading as gg
from pygrading import Job, TestCases
from pygrading.exception import FunctionArgsError

from datetime import datetime


class TestRun(unittest.TestCase):
    def test_Run(self):

        def prework(job: Job):
            print("I am in prework!")

            testcases = gg.create_testcase()
            testcases.append("TestCase1", 20)
            testcases.append("TestCase2", 20)
            testcases.append("TestCase3", 20)
            testcases.append("TestCase4", 20)
            job.set_testcases(testcases)

        def run(job: Job, case: TestCases.SingleTestCase):
            print("I am in run!")
            time.sleep(0.05)
            return "123"

        def postwork(job):
            print("I am in postwork!")
            results = job.get_summary()

            for i in results:
                print(i)

        job = gg.Job(prework=prework, run=run, postwork=postwork)

        start_time = datetime.now()
        job.start(4)
        end_time = datetime.now()
        time_with_thread = (end_time - start_time).total_seconds()

        start_time = datetime.now()
        job.start()
        end_time = datetime.now()
        time_without_thread = (end_time - start_time).total_seconds()

        print(f"time_with_thread: {time_with_thread}")
        print(f"time_without_thread: {time_without_thread}")

        if time_without_thread // time_with_thread < 3:
            raise Exception("多线程模式启动失败")

        job.print()
        print("测试任务基本运行通过")

    def test_prework_agrs(self):

        def prework():
            print("I am in prework!")

        job = gg.Job(prework=prework)
        job.start()
        print("Prework空参数测试通过")

        def prework(job: Job):
            print("I am in prework!")

        job = gg.Job(prework=prework)
        job.start()
        print("Prework带参数测试通过")

        try:
            def prework(job: Job, asd):
                print("I am in prework!")

            job = gg.Job(prework=prework)
            job.start()
        except FunctionArgsError:
            print("Prework多参数异常测试通过")

    def test_run_args(self):
        def run():
            print("I am in run!")

        job = gg.Job(run=run)
        job.start()
        print("run不带参数测试通过")

        def run(job: Job):
            print("I am in run!" + job.get_total_score())

        job = gg.Job(run=run)
        job.start()
        print("run带job参数测试通过")

        testcase = gg.create_testcase(100)
        testcase.append("test1", 100)

        def run(case: TestCases.SingleTestCase):
            print("I am in run!" + case.name)

        job = gg.Job(run=run)
        job.set_testcases(testcase)
        job.start()
        print("run带test参数测试通过")

        def run(job: Job, case: TestCases.SingleTestCase):
            print("I am in run!" + case.name)

        job = gg.Job(run=run)
        job.set_testcases(testcase)
        job.start()
        print("run带两个参数测试通过")

        try:
            def run(job: Job, case: TestCases.SingleTestCase, qwewqe):
                print("I am in run!" + case.name)

            job = gg.Job(run=run)
            job.set_testcases(testcase)
            job.start()
            print(job.get_summary())
        except FunctionArgsError:
            print("run多参数异常测试通过")

    def test_post_args(self):
        def postwork():
            print("I am in postwork!")

        job = gg.Job(postwork=postwork)
        job.start()
        print("Postwork空参数测试通过")

        def postwork(job: Job):
            print("I am in postwork!")

        job = gg.Job(postwork=postwork)
        job.start()
        print("Postwork带参数测试通过")

        try:
            def postwork(job: Job, asd):
                print("I am in prework!")

            job = gg.Job(postwork=postwork)
            job.start()
        except FunctionArgsError:
            print("Postwork多参数异常测试通过")



"""
    Name: job_base.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: The main process of a grading.
    Coding: UTF-8
"""

import json
import types

from pygrading.testcase import TestCases
from pygrading.exception import FunctionsTypeError, FieldMissingError, DataTypeError


class JobBase(object):
    """ A Job is a work flow, using run() function to handle each testcase. """

    def __init__(
        self,
        prework: types.FunctionType = None,
        run: types.FunctionType = None,
        postwork: types.FunctionType = None,
        testcases: TestCases = TestCases(),
        config: dict = {"debug": False}
    ):
        """Init Job instance"""
        self.set_prework(prework)
        self.set_run(run)
        self.set_postwork(postwork)

        self.__testcases = testcases
        self.__config = config
        self.is_terminate = False

        self.__verdict = "Unknown"
        self.__score = 0
        self.__rank = {"rank": "-1.0"}
        self.__comment = ""
        self.__detail = ""
        self.__secret = ""
        self.__HTML = "enable"

        self.__json = {}
        self.__summary = []

    def set_prework(self, prework: types.FunctionType):
        """ Set prework function for job """

        if not prework:
            self.prework = None
            return

        if not callable(prework):
            raise FunctionsTypeError("The prework object passed in is not of function type!")

        self.prework = prework

    def set_run(self, run: types.FunctionType):
        """ Set run function for job """

        if not run:
            self.run = None
            return

        if not callable(run):
            raise FunctionsTypeError("The run object passed in is not of function type!")

        self.run = run

    def set_postwork(self, postwork: types.FunctionType):
        """ Set postwork function for job """

        if not postwork:
            self.postwork = None
            return

        if not callable(postwork):
            raise FunctionsTypeError("The postwork object passed in is not of function type!")

        self.postwork = postwork

    def verdict(self, src: str):
        self.__verdict = src
        self.update_json()

    def score(self, src: int):
        self.__score = str(src)
        self.update_json()

    def rank(self, src: dict):
        self.__rank = src
        self.update_json()

    def comment(self, src: str):
        self.__comment = src
        self.update_json()

    def detail(self, src: str):
        self.__detail = src
        self.update_json()

    def secret(self, src: str):
        self.__secret = src
        self.update_json()

    def HTML(self, src: str):
        self.__HTML = src
        self.update_json()

    def set_summary(self, summary: list):
        self.__summary = summary

    def get_summary(self):
        return self.__summary

    def set_config(self, config):
        self.__config = config

    def get_config(self):
        return self.__config

    def set_testcases(self, testcases):
        self.__testcases = testcases

    def get_testcases(self):
        return self.__testcases.get_testcases()

    def get_total_score(self):
        ret = 0
        for i in self.__summary:
            if type(i) == dict and "score" in i:
                ret += int(i["score"])
        return ret

    def update_json(self):
        self.__json["verdict"] = str(self.__verdict)

        try:
            self.__score = int(self.__score)
        except ValueError:
            raise DataTypeError("Score field must be integer!")

        self.__json["score"] = str(self.__score)

        if "rank" not in self.__rank:
            raise FieldMissingError("No 'rank' detected in super 'rank' field!")

        for k, v in self.__rank.items():
            try:
                num = float(v)
                self.__rank[k] = str(num)
            except ValueError:
                raise DataTypeError("Fields in 'rank' must be single-float!")

        self.__json["rank"] = self.__rank
        self.__json["HTML"] = str(self.__HTML)

        if self.__comment:
            self.__json["comment"] = str(self.__comment)
        elif "comment" in self.__json:
            del self.__json["comment"]

        if self.__detail:
            self.__json["detail"] = str(self.__detail)
        elif "detail" in self.__json:
            del self.__json["detail"]

        if self.__secret:
            self.__json["secret"] = str(self.__secret)
        elif "secret" in self.__json:
            del self.__json["secret"]

    def get_json(self):
        self.update_json()
        return self.__json

    def print(self, return_str=False):
        """ Print result json to stdout or return a json string """

        self.update_json()
        str_json = json.dumps(self.__json)

        if return_str is True:
            return str_json

        print(str_json)



import pygrading as gg

from pygrading import Job


def postwork(job: Job) -> dict:
    """任务后处理函数

    用于汇总测试执行结果

    参数:
        job: 传入的当前任务对象

    返回值:
        返回值为空，后处理内容需要更新到job对象中
    """

    # 获取执行结果总分
    total_score = job.get_total_score()

    # 更新结果输出字段
    job.verdict("Accepted" if total_score == 100 else "Wrong Answer")
    job.score(total_score)
    job.comment("Mission Complete")

    # 渲染 HTML 模板页面（可选）
    # 被渲染的页面应位于 kernel/templates/html 目录下
    job.detail(gg.render_template("index.html", author="Charles Zhang"))



import unittest
import pygrading as gg

from pygrading import Job
from pygrading import TestCases


class TestExample(unittest.TestCase):
    def test_complex(self):

        import multiprocessing

        cpu_core_num = multiprocessing.cpu_count()

        def prework(job: Job):
            testcase = gg.create_testcase(100)
            testcase.append("TestCase1", 25)
            testcase.append("TestCase2", 25)
            testcase.append("TestCase3", 25)
            testcase.append("TestCase4", 25)

            job.set_testcases(testcase)

        def run(job: Job, case: TestCases.SingleTestCase):
            result = {"verdict": "WA", "score": 0}
            if case.score == 25:
                result["verdict"] = "AC"
                result["score"] = 25

            gg.exec("sleep 1", time_out=2)

            return result

        def postwork(job: Job):

            total_score = job.get_total_score()

            job.score(total_score)
            if total_score == 100:
                job.verdict("AC")

            comment = ""
            for ret in job.get_summary():
                comment += str(ret)

            job.comment(comment)
            job.detail("Well Done in detail!")
            job.secret("Well Done in secret")

        new_job = gg.Job(prework=prework, run=run, postwork=postwork)
        new_job.start(max_workers=cpu_core_num)
        new_job.print()



"""
    Name: exception.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: PyGrading exceptions
    Coding: UTF-8
"""


class DataTypeError(Exception):
    pass


class FunctionsTypeError(Exception):
    pass


class FieldMissingError(Exception):
    pass


class ExecError(Exception):
    pass


class FunctionArgsError(Exception):
    pass



import pygrading as gg

from prework import prework
from run import run
from postwork import postwork


if __name__ == '__main__':
    # 创建任务对象
    job = gg.Job(prework=prework, run=run, postwork=postwork)

    # 启动任务并配置最大线程数
    job.start(max_workers=1)

    # 输出结果JSON串
    job.print()



"""
    Name: render.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: Render a html template support Bootstrap4.
    Coding: UTF-8
"""

from jinja2 import Environment, PackageLoader, select_autoescape


class Render(object):

    def __init__(self, package_name: str, package_path: str):

        self.env = Environment(
            loader=PackageLoader(package_name, package_path),
            autoescape=select_autoescape(['html', 'xml'])
        )

    def render(self, template_name, **args):
        return self.env.get_template(template_name).render(args)


def render_template(template_name="index.html", package_name="templates", package_path="html", **args):

    render = Render(package_name, package_path)

    return render.render(template_name, **args)



"""
    Name: testcase.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: A module to generate testcase instance.
    Coding: UTF-8
"""

import os


class TestCases(object):
    """TestCases

    A TestCases class instance is a container of all test cases,
    each test case will be send to a run() function.

    Attributes:
        __count: An integer count of test cases loaded.
        __cases: An list of test cases.
        __total_score: The total score of all test cases.
    """

    class SingleTestCase(object):
        """SingleTestCase

        A SingleTestCase instance contains whole information of a test case.

        Attributes:
            name: The name of the test case.
            score: The score of the test case.
            input_src: The input content of the test case.
            output_src: The output content of the test case.
            extension: (Optional)The extension content of the test case which is customizable.
        """

        def __init__(self, name: str, score: int, input_src=None, output_src=None, extension=None):
            self.name = str(name)
            self.score = int(score)
            self.input_src = input_src
            self.output_src = output_src
            self.extension = extension

        def __str__(self):
            """Overview single cases."""
            ret = ["{}:\n".format(self.name),
                   "\tscore: {}\n".format(self.score),
                   "\tinput_src: {}\n".format(str(self.input_src)),
                   "\toutput_src: {}\n".format(str(self.output_src)),
                   "\textension: {}\n".format(str(self.extension))]
            return ''.join(ret)

    def __init__(self, count: int = 0, total_score: int = 100):
        """Init TestCases with empty values"""
        self.__count = count
        self.__cases = []
        self.__total_score = total_score

    def __str__(self):
        """Overview test cases."""
        ret = ["Cases Count: {}\n".format(self.__count), "Total Scare: {}\n".format(self.__total_score)]
        for case in self.__cases:
            ret.append("{}:\n".format(case.name))
            ret.append("\tscore: {}\n".format(case.score))
            ret.append("\tinput_src: {}\n".format(str(case.input_src)))
            ret.append("\toutput_src: {}\n".format(str(case.output_src)))
            ret.append("\textension: {}\n".format(str(case.extension)))
        return ''.join(ret)

    def get_count(self):
        return self.__count

    def get_total_score(self):
        return self.__total_score

    def get_testcases(self):
        return self.__cases

    def set_total_score(self, total_score: int):
        self.__total_score = total_score

    def isempty(self):
        return self.__count == 0

    def append(self, name: str, score: int, input_src=None, output_src=None, extension=None):
        """Add test case to list of test cases."""
        single_case = self.SingleTestCase(name, score, input_src, output_src, extension)
        self.__cases.append(single_case)
        self.__count += 1


def create_testcase(total_score: int = 100):
    """
    Create a empty TestCases instance

    Args:
        total_score: The total score of all test cases.

    Returns:
        test_cases: A instance of TestCases.
    """
    return TestCases(total_score=total_score)


def create_std_testcase(testcase_dir: str, testcase_num: int):
    """
    Create test cases in a recommended way.
    For example:
        testdata
        ├── input
        │   ├── input1.txt
        │   ├── input2.txt
        │   └── input3.txt
        └── output
            ├── output1.txt
            ├── output2.txt
            └── output3.txt

    Returns:
        test_cases: A instance of TestCases.

    Raises:
        IOError: A error occurred when missing file.
    """
    test_cases = TestCases()

    # Fix typing error
    testcase_num = int(testcase_num)

    for i in range(1, testcase_num + 1):

        input_scr = os.path.join(testcase_dir, "input", "input{}.txt".format(i))
        output_src = os.path.join(testcase_dir, "output", "output{}.txt".format(i))

        if not os.path.exists(input_scr):
            raise IOError("".join(["Missing file ", input_scr]))

        if not os.path.exists(output_src):
            raise IOError("".join(["Missing file ", output_src]))

        test_cases.append(
            name="TestCase{}".format(i),
            score=(test_cases.get_total_score() / testcase_num),
            input_src=input_scr,
            output_src=output_src
        )

    return test_cases



"""
    Name: html.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: A module to generate html code.
    Coding: UTF-8
"""


def str2html(src: str) -> str:
    """Switch normal string to a html type"""
    if not src:
        return ""
    str_list = src.split("\n")
    while str_list[-1] == "\n" or str_list[-1] == "":
        str_list.pop()

    for i in range(len(str_list) - 1):
        str_list[i] += "<br>"

    return "".join(str_list)


class SingleTag(object):
    """Tag

    A super class of all single html tag class.

    Attributes:
        name: Tag name.
        attributes: Attributes of this tag.
    """

    def __init__(self, **attributes):
        self.name = "tag"
        self.attributes = attributes

    def __str__(self):
        ret = ["".join(["<", self.name])]
        for key, value in self.attributes.items():
            ret.append("".join([" ", key, "=", "'", value, "'"]))
        ret.append(">")

        return "".join(ret)

    def print(self):
        print(str(self))


class Tag(object):
    """Tag

    A super class of all pair html tag class.

    Attributes:
        text: Text content.
        name: Tag name.
        subtag: Sub tags.
        attributes: Attributes of this tag.
    """

    def __init__(self, *subtag, **attributes):
        self.text = ""
        self.name = "tag"

        for tag in subtag:
            if (not isinstance(tag, Tag)) and (not isinstance(tag, SingleTag)):
                raise ValueError(str(tag) + "is not a subtag.")

        self.subtag = []
        for tag in subtag:
            self.subtag.append(tag)

        self.attributes = attributes

    def __str__(self):
        ret = []

        pre = ["".join(["<", self.name])]
        for key, value in self.attributes.items():
            pre.append("".join([" ", key, "=", "'", value, "'"]))
        pre.append(">")
        ret.append("".join(pre))

        if self.text:
            ret.append(self.text)

        for s in self.subtag:
            ret.append(str(s))

        ret.append("".join(["</", self.name, ">"]))

        return "".join(ret)

    def __lshift__(self, other):
        if not isinstance(other, Tag):
            raise ValueError(str(other) + "is not a subtag.")
        self.subtag.append(other)
        return self

    def append(self, obj):
        if not isinstance(obj, Tag):
            raise ValueError(str(obj) + "is not a subtag.")
        self.subtag.append(obj)

    def pop(self, index=-1):
        self.subtag.pop(index)

    def insert(self, index, obj):
        if not isinstance(obj, Tag):
            raise ValueError(str(obj) + "is not a subtag.")
        self.subtag.insert(index, obj)

    def extend(self, seq):
        for obj in seq:
            if not isinstance(obj, Tag):
                raise ValueError(str(obj) + "is not a subtag.")
        self.subtag.extend(seq)

    def print(self):
        print(str(self))

    def set_text(self, src: str):
        self.text = src
        return self


class custom_single_tag(SingleTag):
    def __init__(self, tag_name, **attributes):
        super().__init__(**attributes)
        self.name = tag_name


class br(SingleTag):
    def __init__(self, **attributes):
        super().__init__(**attributes)
        self.name = "br"


class img(SingleTag):
    def __init__(self, **attributes):
        super().__init__(**attributes)
        self.name = "img"


class input_tag(SingleTag):
    def __init__(self, **attributes):
        super().__init__(**attributes)
        self.name = "input"


class custom(Tag):
    def __init__(self, tag_name, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = tag_name


class a(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "a"


class body(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "body"


class div(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "div"


class font(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "font"


class form(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "form"


class h1(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "h1"


class h2(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "h2"


class h3(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "h3"


class h4(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "h4"


class h5(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "h5"


class h6(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "h6"


class head(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "head"


class html(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "html"


class p(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "p"


class table(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "table"


class th(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "th"


class title(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "title"


class tr(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "tr"


class td(Tag):
    def __init__(self, *subtag, **attributes):
        super().__init__(*subtag, **attributes)
        self.name = "td"



import unittest
import pygrading as gg

from pygrading.exception import FunctionsTypeError, DataTypeError, FieldMissingError


class TestJob(unittest.TestCase):
    def test_job_create(self):
        try:
            gg.Job(prework=123)
        except gg.exception.FunctionsTypeError as e:
            print("捕获传入函数类型错误成功！" + str(e))

        try:
            gg.Job(run="1231")
        except gg.exception.FunctionsTypeError as e:
            print("捕获传入函数类型错误成功！" + str(e))

        try:
            gg.Job(postwork=True)
        except gg.exception.FunctionsTypeError as e:
            print("捕获传入函数类型错误成功！" + str(e))

        try:
            gg.Job(prework=None, run=None, postwork=None)
            print("Job接收空参数任务成功")
        except FunctionsTypeError as e:
            raise e

        def prework():
            print("I am from prework!")

        def run():
            print("I an from run!")

        def postwork():
            print("I am from postwork")

        try:
            gg.Job(prework=prework, run=run, postwork=postwork)
            print("Job接收正常参数任务成功")
        except FunctionsTypeError as e:
            raise e

    def test_job_json(self):
        job = gg.Job()
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Unknown", "score": "0", "rank": {"rank": "-1.0"}, "HTML": "enable"}"""
        )

        job.verdict("Accepted")
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "0", "rank": {"rank": "-1.0"}, "HTML": "enable"}"""
        )

        job.score(100)
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, "HTML": "enable"}"""
        )

        try:
            job.score(100.00)
        except DataTypeError:
            print("检测score字段为浮点数的错误成功")
            job.score(100)

        job.rank({"rank": "-1", "test1": "0.5"})
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0", "test1": "0.5"}, "HTML": "enable"}"""
        )

        try:
            job.rank({"test1": "0.5"})
        except FieldMissingError:
            print("捕获缺失rank值错误成功！")

        try:
            job.rank({"rank": "-1", "test1": "hello"})
        except DataTypeError:
            print("捕获rank值不为浮点数错误成功！")

        job.rank({"rank": "-1"})
        job.comment("test")
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, "HTML": "enable", "comment": "test"}"""
        )

        job.comment("123")
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, "HTML": "enable", "comment": "123"}"""
        )
        print("正确将comment中数字类型转化为字符串")

        job.comment("")
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, "HTML": "enable"}"""
        )
        print("正确删除comment字段")

        job.detail("test")
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, "HTML": "enable", "detail": "test"}"""
        )

        job.detail("123")
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, "HTML": "enable", "detail": "123"}"""
        )
        print("正确将detail中数字类型转化为字符串")

        job.detail("")
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, "HTML": "enable"}"""
        )
        print("正确删除detail字段")

        job.secret("test")
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, "HTML": "enable", "secret": "test"}"""
        )

        job.secret("123")
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, "HTML": "enable", "secret": "123"}"""
        )
        print("正确将secret中数字类型转化为字符串")

        job.secret("")
        job.print()

        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, "HTML": "enable"}"""
        )
        print("正确删除secret字段")

        job.comment("test")
        job.detail("test")
        job.secret("test")
        job.print()
        self.assertEqual(
            job.print(return_str=True),
            """{"verdict": "Accepted", "score": "100", "rank": {"rank": "-1.0"}, """ +
            """"HTML": "enable", "comment": "test", "detail": "test", "secret": "test"}"""
        )
        print("正确添加所有字段字段")



import pygrading as gg

from pygrading import Job, TestCases


def run(job: Job, case: TestCases.SingleTestCase) -> dict:
    """任务执行函数

    用于执行测试用例

    参数:
        job: 传入的当前任务对象
        case: 单独的测试用例对象

    返回值:
        result: 测试用例执行结果字典，该结果会被汇总到job对象的summary对象中
    """

    # 创建一个结果对象
    result = {"name": case.name, "score": 0}

    # 执行评测任务命令
    exec_result = gg.exec("echo success")

    if gg.utils.compare_str(exec_result.stdout, "success"):
        result["score"] = case.score

    return result



import pygrading as gg

from pygrading import Job


def prework(job: Job):
    """任务预处理函数

    通常用于创建测试用例和创建配置信息

    参数:
        job: 传入的当前任务对象

    返回值:
        返回值为空，预处理内容需要更新到job对象中
    """

    # 创建测试用例和配置信息
    config = {"debug": False}

    testcases = gg.create_testcase(100)
    testcases.append(name="TestCase1", score=50, input_src="", output_src="")
    testcases.append(name="TestCase2", score=50, input_src="", output_src="")

    # 更新测试用例和配置信息到任务
    job.set_config(config)
    job.set_testcases(testcases)



"""
    Name: __main__.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: Main funcution for console application
    Coding: UTF-8
"""

import fire
import pygrading as gg

from pygrading.init_project import init


def version():
    print(gg.__version__)


if __name__ == "__main__":
    fire.Fire({
        "-v": version,
        "version": version,
        "init": init
    })



"""
    Name: job_run.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: Job running process.
    Coding: UTF-8
"""

import inspect

from concurrent.futures import ThreadPoolExecutor
from pygrading.job_base import JobBase
from pygrading.exception import FunctionArgsError


class Job(JobBase):
    """ Running a job and support multi-threading """

    def start(self, max_workers: int = 1):
        """ start a job """

        # run prework
        if self.prework:
            prework_inspect = inspect.getfullargspec(self.prework)

            if not prework_inspect.args:
                self.prework()
            elif len(prework_inspect.args) == 1:
                self.prework(self)
            else:
                raise FunctionArgsError("Prework function supports\n\t1. prework()\n\t2. prework(job: Job)")

        if self.is_terminate:
            return self.print(return_str=True)

        # get testcases and bind with job obj
        testcases = [(self, case) for case in self.get_testcases()]

        # create a thradd pool
        def outer_run(args):
            """
            support these four situations:
            1. run(job, testcase)
            2. run(testcase)
            3. run(job)
            4. run()
            """
            run_inspect = inspect.getfullargspec(self.run)

            if not run_inspect.args:
                return self.run()
            elif len(run_inspect.args) == 2:
                return self.run(*args)
            elif len(run_inspect.args) == 1:
                if run_inspect.args[0] == "job":
                    return self.run(args[0])
                elif run_inspect.args[0] == "case":
                    return self.run(args[1])
            else:
                raise FunctionArgsError(
                    "Run function supports\n\t" +
                    "1. run()\n\t" +
                    "2. run(job: Job)\n\t" +
                    "3. run(case: TestCases.SingleTestCase)\n\t" +
                    "4. run(job: Job, case: TestCases.SingleTestCase)\n\t"
                )

        if self.run:
            with ThreadPoolExecutor(max_workers=max_workers, thread_name_prefix="job_") as pool:
                self.set_summary([result for result in pool.map(outer_run, testcases)])

        # run postwork
        if self.postwork:
            postwork_inspect = inspect.getfullargspec(self.postwork)

            if not postwork_inspect.args:
                self.postwork()
            elif len(postwork_inspect.args) == 1:
                self.postwork(self)
            else:
                raise FunctionArgsError("Postwork function supports\n\t1. prework()\n\t2. prework(job: Job)")

        return self.print(return_str=True)



"""
    Name: utils.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: A tool box contains many frequently used functions.
    Coding: UTF-8
"""

import os
import time
import shutil
import base64
import subprocess

from pygrading.html import img
from pygrading.exception import ExecError


class Exec(object):
    """Exec

    A Exec class instance contains all exec result.

    Attributes:
        cmd: Exec command.
        stdout: Standard output.
        stderr: Standard error.
        exec_time: Execute time.
        returncode: Return code.
    """

    def __init__(self, cmd: str, stdout: str, stderr: str, exec_time: int, returncode: int):
        self.cmd = cmd
        self.stdout = stdout
        self.stderr = stderr
        self.exec_time = exec_time
        self.returncode = returncode


def exec(cmd: str, input_str: str = "", time_out: int = None, queit: bool = True, env: dict = None) -> Exec:
    """Run a shell command with bash

    Args:
        cmd: A command string.
        input_str: Input resources for stdin.

    Returns:
        exec_result: A Exec instance.
    """
    begin = time.time()
    process = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                               encoding="utf8", env=env)
    out, err = process.communicate(input_str, timeout=time_out)
    end = time.time()
    exec_time = int((end - begin) * 1000)

    if process.returncode != 0 and not queit:
        raise ExecError(f"Error occurred when running command: {cmd}\n" + err)

    return Exec(cmd, out, err, exec_time, process.returncode)


def makedirs(path: str, exist_ok: bool = False):
    """ Make Dirs """
    os.makedirs(path, exist_ok=exist_ok)


def copyfile(src: str, dst: str):
    """Copy File"""
    shutil.copyfile(src, dst)


def copytree(src: str, dst: str):
    """Copy Tree"""
    shutil.copytree(src, dst)


def move(src: str, dst: str):
    """Move File"""
    shutil.move(src, dst)


def rmtree(src: str):
    """Remove Tree"""
    shutil.rmtree(src)


def rmfile(src: str):
    """Remove File"""
    os.remove(src)


def rename(old: str, new: str):
    """Rename File"""
    os.rename(old, new)


def writefile(src: str, lines: str, option: str = "w"):
    """ Write string to file """
    with open(src, option, encoding='utf-8') as f:
        f.write(lines)


def readfile(src: str) -> list:
    """
    Read file and return list.
    Auto remove blank line at the end of the file
    """
    with open(src, 'r', encoding='utf-8') as f:
        lines = f.readlines()

        for i in range(len(lines)):
            lines[i] = lines[i].rstrip('\n')

        while "" == lines[-1]:
            lines.pop()

        return lines


def writefile_list(src: str, lines: list, option: str = "w"):
    """ Write string list to file """
    with open(src, option, encoding='utf-8') as f:
        f.writelines(lines)


def str2list(src: str) -> list:
    """Separate a string to a list by \n and ignore blank line at the end automatically."""
    ret = src.split("\n")
    while ret[-1] == "\n" or ret[-1] == "":
        ret.pop()
    return ret


def compare_str(str1: str, str2: str) -> bool:
    """Compare two string and ignore \n at the end of two strings."""
    return str1.rstrip() == str2.rstrip()


def compare_list(list1: list, list2: list) -> bool:
    """Compare two list and ignore \n at the end of two list."""
    while list1[-1] == "\n" or list1[-1] == "":
        list1.pop()
    while list2[-1] == "\n" or list2[-1] == "":
        list2.pop()
    return list1 == list2


def edit_distance(obj1, obj2) -> int:
    """Calculates the edit distance between obj1 and obj2"""
    edit = [[i + j for j in range(len(obj2) + 1)] for i in range(len(obj1) + 1)]

    for i in range(1, len(obj1) + 1):
        for j in range(1, len(obj2) + 1):
            if obj1[i - 1] == obj2[j - 1]:
                d = 0
            else:
                d = 1
            edit[i][j] = min(edit[i - 1][j] + 1, edit[i][j - 1] + 1, edit[i - 1][j - 1] + d)

    return edit[len(obj1)][len(obj2)]


def get_result_lines(stdout, tag) -> list:
    """ Get lines contains tag from stdout """
    return [raw for raw in stdout.split("\n") if tag in raw]


def get_result_dict(stdout, separater="=") -> list:
    """ Analyze stdout make it to a dict """
    lines, dic = [line for line in stdout.split("\n") if separater in line], {}

    for line in lines:
        key_val = line.split(separater)
        if len(key_val) != 2:
            continue
        else:
            dic[key_val[0].strip()] = key_val[1].strip()

    return dic


def img2base64(path):
    """ Transfore image to base64 string """
    image = "data:image/jpg;base64,"
    if os.path.exists(path):
        with open(path, "rb") as f:
            image += str(base64.b64encode(f.read()))[2:-1]

    return image


def get_image_element(path):
    """ Return a base64 image element """
    src = img2base64(path)
    return img(src=src, width="256")


def loge(config: dict, message: str, key: str = "debug"):
    """ Print debug log to stdout """

    if key not in config or not config[key]:
        return

    else:
        print(message)



"""
    Name: init_project.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: Initialize a pygrading project
    Coding: UTF-8
"""


import os
import pkgutil
from pygrading.utils import makedirs, writefile


def copy_file(pkg_path, target_path, name):
    data = pkgutil.get_data(__package__, pkg_path).decode()
    writefile(os.path.join(target_path, name), str(data))
    print(f"copy {pkg_path} done.")


def init(path=os.getcwd(), name="cg-kernel"):
    """ Initialize a pygrading project """

    print("Create project folder", end="")
    new_project_path = os.path.join(path, name)
    new_project_kernel_path = os.path.join(new_project_path, "kernel")
    new_project_template_path = os.path.join(new_project_kernel_path, "templates")

    html_template_path = os.path.join(new_project_template_path, "html")

    makedirs(new_project_template_path, exist_ok=True)
    makedirs(html_template_path, exist_ok=True)
    print("Create project folder success")

    print("Copy files start ===")
    copy_file("static/README.md", new_project_path, "README.md")
    copy_file("static/Makefile", new_project_path, "Makefile")
    copy_file("static/.gitignore", new_project_path, ".gitignore")
    copy_file("static/kernel/__init__.py", new_project_kernel_path, "__init__.py")
    copy_file("static/kernel/__main__.py", new_project_kernel_path, "__main__.py")
    copy_file("static/kernel/prework.py", new_project_kernel_path, "prework.py")
    copy_file("static/kernel/run.py", new_project_kernel_path, "run.py")
    copy_file("static/kernel/postwork.py", new_project_kernel_path, "postwork.py")
    copy_file("static/kernel/templates/__init__.py", new_project_template_path, "__init__.py")
    copy_file("static/kernel/templates/html/base.jinja", html_template_path, "base.jinja")
    copy_file("static/kernel/templates/html/index.html", html_template_path, "index.html")
    print("Copy files success")

    print("Initialization Completed!")



'''
    Name: setup.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: Setup script for pygrading!
    Coding: UTF-8
'''

# python setup.py sdist bdist_wheel
# python -m twine upload dist/*

import setuptools

from pygrading import __version__

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='pygrading',
    version=__version__,
    author='Charles Zhang',
    author_email='694556046@qq.com',
    description='A Python ToolBox for CourseGrading platform.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://gitlab.educg.net/zhangmingyuan/PyGrading',
    packages=['pygrading'],
    package_data={
        'pygrading': [
            'static/.gitignore',
            'static/*',
            'static/kernel/*',
            'static/kernel/templates/*',
            'static/kernel/templates/html/*',
        ],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.5',
    install_requires=[
        'Jinja2>=2.11.2',
        'fire>=0.3.1'
    ],
)



"""
    Name: __init__.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: A Python ToolBox for CourseGrading platform
    Coding: UTF-8
"""

from pygrading.job import Job
from pygrading.utils import exec, loge
from pygrading.render import render_template
from pygrading.configuration import load_config
from pygrading.testcase import create_testcase, create_std_testcase, TestCases

__all__ = ["Job", "load_config", "create_testcase", "create_std_testcase", "TestCases",
           "exec", "loge", "render_template"]
__version__ = "v1.1.5"



import unittest
import pygrading as gg
from subprocess import TimeoutExpired
from pygrading.exception import ExecError


class TestExec(unittest.TestCase):
    def test_exec(self):
        try:
            gg.exec("sleep 2", time_out=1)
        except TimeoutExpired:
            print("exec超时功能正常")

        gg.exec("asdsad", queit=True)
        print("exec静默模式正常")

        try:
            gg.exec("asdsad", queit=False)
        except ExecError as e:
            print("exec关闭静默模式正常")
            print(e)

    def test_dict(self):
        stdout = "a=1\nb = 2\nc=3\neafafd"
        dic = gg.utils.get_result_dict(stdout, "=")
        print(dic)



import unittest
import pygrading as gg


class TestImport(unittest.TestCase):
    def test_import(self):
        try:
            print(gg.__version__)
        except Exception:
            raise AssertionError("Import PyGrading Failed!")
